﻿using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Layouts;
using ArcGIS.Desktop.Mapping;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Security.Policy;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace TaxParcelPlenary
{
  internal class DisasterTsunami : Button
  {
    protected override async void OnClick()
    {
      if (MapView.Active == null) return;
      try
      {
        // First make the tsunami layer visible
        var tsunamiLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals("SouthShoreImpact")).FirstOrDefault();
        if (tsunamiLayer == null) return;

        // activate the Tax table view as the active view
        var taxTable = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(Module1.TaxParcelPolygonLayerName)).FirstOrDefault();
        if (taxTable == null) return;

        // Set the Tax_Lines as un-selectable layer and taxtable as selectable
        var taxLine = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(Module1.TaxParcelLineLayerName)).FirstOrDefault();
        if (taxLine == null) return;

        await QueuedTask.Run(() =>
        {
          taxLine.SetSelectable(false);
          taxLine.ClearSelection();
          taxTable.SetScaleSymbols(true);
          taxTable.ClearSelection();
        });

        // run on UI thread
        var iTableView = Module1.OpenAndActivateTablePane(taxTable);
        if (iTableView == null) return;

        await QueuedTask.Run(() =>
        {
          tsunamiLayer.SetVisibility(!tsunamiLayer.IsVisible);

          // select all tax records that are within the SouthShoreImpact zone
          // first get the complete SouthShoreImpact geometry
          var rowCursorOverlayPoly = tsunamiLayer.Search();
          Geometry disasterArea = null;
          while (rowCursorOverlayPoly.MoveNext())
          {
            using var feature = rowCursorOverlayPoly.Current as Feature;
            if (disasterArea == null)
            {
              disasterArea = feature.GetShape().Clone();
              continue;
            }
            disasterArea = GeometryEngine.Instance.Union(disasterArea, feature.GetShape().Clone());
          }
          var description = string.Empty;
          if (disasterArea.IsEmpty) return;

          // define the spatial query filter
          // exclude roads
          var spatialQuery = new SpatialQueryFilter()
          {
            WhereClause = "not (name like '%-999')",
            FilterGeometry = disasterArea,
            SpatialRelationship = SpatialRelationship.Intersects
          };
          // which tax polygons are selected
          (taxTable as FeatureLayer).Select(spatialQuery);

          // zoom the map to the selection
          if (MapView.Active != null)
            MapView.Active.ZoomToSelected(); // .ZoomTo(disasterArea.Extent.Expand(1.1,1.1,true));
        });
        
        await QueuedTask.Run(() =>
        {
          Module1.OpenAndActivateTablePane(taxTable);
        });
        var tableView = TableView.Active;
        if (tableView == null) return;
        if (tableView.IsReady)
        {
          // set visible and hidden columns
          tableView.ShowAllFields();
          // adds to hidden
          tableView.SetHiddenFields(new List<string> { "GlobaID", "CREATEDBYRECORD", "RETIREDBYRECORD", "CalculatedArea", "MiscloseRatio", "MiscloseDistance", "IsSeed", "created_user", "created_date", "last_edited_user", "last_edited_date", "Shape_Length", "Shape_Area", "VALIDATIONSTATUS", "Parcel", "TMK", "qpub_link", "Zone", "Section", "Plat", "Island" });
          // clear and reset frozen fields
          var frozenFields = tableView.GetFrozenFields();
          if (!frozenFields.Contains("Name"))
          {
            await tableView.ClearAllFrozenFieldsAsync();
            await tableView.SetFrozenFieldsAsync(new List<string> { "OBJECTID", "Name" });
          }
          await tableView.SetViewMode(TableViewMode.eSelectedRecords);
          tableView.SetZoomLevel(150);
        }

        // activate condition so we can see CAMA buttons on the Ribbon
        Module1.ChangeConditionState("state_DisasterOverlay", true);
        Module1.ChangeConditionState("state_AddToCAMA", true);
      }
      catch (Exception ex)
      {
        MessageBox.Show($@"Error: {ex.Message}");
      }
    }

  }
}
